@@ -189,6 +189,15 @@ impl JobOpt {
             Self::Python(x) => &x.job_name,
         }
     }
+
+    #[cfg(test)]
+    pub(crate) fn placeholder_test_data() -> Self {
+        transport::JobOpt::Python(transport::PythonJob {
+            python_file: vec![],
+            job_name: "test_job".into(),
+            job_files: vec![],
+        })
+    }
 }
 
 #[cfg(feature = "cli")]
@@ -661,6 +670,7 @@ mod tests {
     }
 
     #[test]
+    #[serial_test::serial]
     /// ensure the length of data in the filesystem is the length of the
     /// data if we were to read it into a vector
     fn check_filesystem_bytes() {
@@ -704,6 +714,7 @@ mod tests {
     }
 
     #[tokio::test]
+    #[serial_test::serial]
     async fn read_from_writer_binary() {
         //crate::logger();
         let len = 1099;
